////////////////////////////////////////////////////////////
//                  *.ini                  //
//                        v 1.1                           //
//                                                        //
//              Copyright (C) 2007 Sagrer                 //
//             LGPL 2.1         //
//                  .  lgpl.txt                     //
//                                                        //
//                  sagrer@yandex.ru                      //
////////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

unit KOLAnIniFile;

interface
uses KOL, KOLFileInString, KOLTextFileInString;

type
  TAnIniFile = class (TObject)
    Public
      TextFil1 : TTextFileInString;
      TextFil2 : TTextFileInString;
      Constructor Init;
      Function Load(const FileName : string) : boolean;
      Function Save(const FileName : string) : boolean;
      Function SectionExists(const SectName : string) : boolean;
      Function ElementExists(const SectName, ElemName : string) : boolean;
      Function ReadString(const SectName, ElemName : string) : string;
      Function ReadInteger(const SectName, ElemName : string) : integer;
      Function ReadFloat(const SectName, ElemName : string) : double;
      Function ReadBool(const SectName, ElemName : string) : boolean;
      Procedure WriteString(const SectName, ElemName, Str : string);
      Procedure WriteInteger(const SectName, ElemName : string; Int : integer);
      Procedure WriteFloat(const SectName, ElemName : string; Flt : double);
      Procedure WriteBool(const SectName, ElemName : string; Bool : boolean);
      Procedure Kill;
      Procedure MakNewFile;
    Private
      Function GoToSection(const SectName : string; TypePos : byte) : boolean;
      Function GoToElement(const SectName, ElemName : string) : boolean;
      Function NotComment(const Str : string) : boolean;
      Function GetElemName(const Str : string) : string;
      Function GetElemStr(const Str : string) : string;
      Function CheckStrInt(const St : string) : boolean;
      Function CheckCharNum(const Ch : char) : boolean;
      Function CheckStrFloat(const St : string) : boolean;
      
  end;

implementation

Constructor TAnIniFile.Init;
begin
  //.   .
  TextFil1 := TTextFileInString.init;
  TextFil2 := TTextFileInString.init;
end;

Procedure TAnIniFile.Kill;
begin
  // .
  TextFil1.Free;
  TextFil2.Free;
  Free;
end;

Function TAnIniFile.Load(const FileName : string) : boolean;
begin
  //    -.
  TextFil1.Load(FileName);
  Result := true;
end;

Function TAnIniFile.Save(const FileName : string) : boolean;
begin
  //  -  .
  TextFil1.Save(FileName);
  Result := true;
end;

Function TAnIniFile.SectionExists(const SectName : string) : boolean;
var
  TempCur : int64;
begin
  //,    .     :)

  TempCur := TextFil1.Cursor;
  Result := GoToSection(SectName,1);
  TextFil1.Cursor := TempCur;

end;

Function TAnIniFile.ElementExists(const SectName, ElemName : string) : boolean;
var
  TempCur : int64;
begin
  //,    .    .

  TempCur := TextFil1.Cursor;
  Result := GoToElement(SectName,ElemName);
  TextFil1.Cursor := TempCur;
end;

Function TAnIniFile.ReadString(const SectName, ElemName : string) : string;
begin
  //      .
  Result := '';   //    :)

  //   .
  If GoToElement(SectName,ElemName) = true then begin
    // -     (   ).
    Result := GetElemStr(TextFil1.ReadStrLn(TextFil1.Cursor));
  end;
end;

Function TAnIniFile.ReadInteger(const SectName, ElemName : string) : integer;
var
  TempStr : string;
begin
  //      .
  TempStr := ReadString(SectName,ElemName);

  //,    .
  If CheckStrInt(TempStr) = true then begin
    // - ,   ...
    Result := Str2Int(TempStr);
  end
  else begin
    //  - ,   
    Result := 0;
  end;
end;

Function TAnIniFile.ReadFloat(const SectName, ElemName : string) : double;
var
  TempStr : string;
begin
  //      .
  TempStr := ReadString(SectName,ElemName);

  //,    .
  If CheckStrFloat(TempStr) = true then begin
    // - ,   ...
    Result :=Str2Double(TempStr);
  end
  else begin
    //  - ,   
    Result := 0;
  end;
end;

Function TAnIniFile.ReadBool(const SectName, ElemName : string) : boolean;
var
  TempStr : string;
  TempInt : integer;
begin
  //      .
  //   ,   
  //  .
  TempStr := ReadString(SectName,ElemName);

  //,    .
  If CheckStrInt(TempStr) = true then begin
    // - ,   ...
    TempInt := Str2Int(TempStr);
  end
  else begin
    //  - ,   
    TempInt := 0;
  end;

  //    .
  If TempInt = 1 then begin
    Result := true;
  end
  else begin
    Result := false;
  end;
end;

Procedure TAnIniFile.WriteString(const SectName, ElemName, Str : string);
begin
  //   -   ,   ,
  //     .    ,  
  //,       .
  //     .    
  // .  (    )
  //   ,   ...

  //,   - ,   .
  If GoToElement(SectName,ElemName) = true then begin
    //  .    ...
    TextFil1.ReplaceString(TextFil1.Cursor,ElemName+'='+Str);
  end
  else begin
    //   .  ,      ...
    If GoToSection(SectName,3) = true then begin
      // ,    ,      ...
    end
    else begin
      //,      :).
      //      
      //   ...

      If TextFil1.Siz <> 0 then begin
        //,     .
        TextFil1.Cursor := TextFil1.Siz+1;    //   ...
        If TextFil1.FileString[TextFil1.Cursor-1] = #10 then begin
          //   -   
          //    
          //  1   ,       .
          //           
          //   #13#10     .
          If TextFil1.Siz > 3 then begin
            If TextFil1.FileString[TextFil1.Cursor-3] <> #10 then begin
              TextFil1.AddStr('');
            end;
          end;
          TextFil1.AddStr('['+SectName+']');
          TextFil1.Cursor := TextFil1.Siz+1;  //    .
        end
        else begin
          //  -    ( )
          // -   ,  ,    
          TextFil1.AddStr(#13+#10+#13+#10+'['+SectName+']'); //     
                                    //   ,     
          TextFil1.Cursor := TextFil1.Siz+1;  //    .
        end;
      end
      else begin
        //   ,       :))
        TextFil1.AddStr('['+SectName+']');
        TextFil1.Cursor := TextFil1.Siz+1;  //    .
      end;  
    end;

    //  ,     ,     
    //   -       .
    //       (..    
    //,     ...

    TextFil1.InsertString(TextFil1.Cursor,ElemName+'='+Str+#13+#10);

  end;
end;

Procedure TAnIniFile.WriteInteger(const SectName, ElemName : string; Int : integer);
begin
  //   ,    .
  WriteString(SectName,ElemName,Int2Str(Int));
end;

Procedure TAnIniFile.WriteFloat(const SectName, ElemName : string; Flt : double);
begin
  //   ,    .
  WriteString(SectName,ElemName,Double2Str(Flt));
end;

Procedure TAnIniFile.WriteBool(const SectName, ElemName : string; Bool : boolean);
var
  TempInt : Integer;
begin
  //  ,    (  
  //     ).

  // 
  If Bool = true then begin
    //true
    TempInt := 1;
  end
  else begin
    //false
    TempInt := 0;
  end;

  //
  WriteInteger(SectName,ElemName,TempInt);    
end;

Function TAnIniFile.GoToSection(const SectName : string; TypePos : byte) : boolean;
var
  buf, TestStr : string;
  Finded, TempBool1 : boolean;
  SectLength, I, SectStartCur, TempCur : integer;
begin
  //  -      :)
  
  //     ,   
  //  -  ,   - .
  //      ,    
  //  - .
  result := false;
  Finded := false;
  SectLength := Length(SectName)+2;

  //  -    .
  // - ,   -    :)

  If TextFil1.Siz > 0 then begin
    TextFil1.Cursor := 1;
    repeat
      //  (       ).
      SectStartCur := TextFil1.Cursor;  //     
      buf := TextFil1.CurReadStr;
      //    ...
      If Length(buf) > 0 then begin
        // -    
        If buf[1] = '[' then begin
          //  - .
          //  (  ,  )
          If Length(buf) >= SectLength then begin
            //    
            TestStr := '';
            For I := 1 to SectLength do begin
              TestStr := TestStr+buf[I];
            end;

            // -     -.
            If UpperCase('['+SectName+']') = UpperCase(TestStr) then begin
              //  .  .
              Finded := true;
            end;
          end;
        end;
      end;
      If (TextFil1.Cursor <> Length(TextFil1.FileString)+1) and ((Length(TextFil1.FileString)+1)  - (TextFil1.Cursor) >=2)   then begin
        //     ,    2 , ...
        TextFil1.Cursor := TextFil1.Cursor+2;
      end;
    until (Finded = true) or (TextFil1.Eof);

    //  -     , 
    //    (,    
    //).

    If Finded = true then begin
      //    TypePos
      // 1 -    
      // 2 -    
      // 3 -    (    )

      If TypePos = 1 then begin
        //   
        TextFil1.Cursor := SectStartCur;
        Result := true;
      end;

      If TypePos = 2 then begin
        //   
        TextFil1.Cursor := SectStartCur;
        TextFil1.CurReadLn;  //    .
        //     ,  " " -   
        //  .
        if TextFil1.Eof = false then begin
          If TextFil1.FileString[TextFil1.Cursor] <> '[' then begin
            Result := true;
          end
          else begin
            //     ,      :)
            TextFil1.Cursor := SectStartCur;
            TextFil1.CurReadStr;       //   - 
            TextFil1.InsertString(TextFil1.Cursor,#13#10);
            TextFil1.Cursor := TextFil1.Cursor+2;
          end;
        end;
      end;

      If TypePos = 3 then begin
        //   
        TextFil1.Cursor := SectStartCur;
        TempCur := TextFil1.Cursor;
        //TextFil1.CurReadLn;   //    .
        //       .
        repeat
          TempBool1 := false;
          //TempCur := SectStartCur;  //   -  
                                    // 
          If TextFil1.Eof <> true then begin
            TempCur := TextFil1.Cursor;  //  "" 
            TextFil1.CurReadLn;  //  . .
            buf := TextFil1.ReadStrLn(TextFil1.Cursor);   // 1  ...
            If Length(buf) > 0 then begin
              // -    .
              //..    ,     
              //. (      ).
              If buf[1] = '[' then begin
                TempBool1 := true; //  
                //TempCur :=
              end;
            end;
          end;
        until  (TempBool1 = true) or (TextFil1.Eof = true);

        //,     ""
        If TextFil1.Eof = true then begin
          //,       .
          if TextFil1.FileString[TextFil1.Cursor-1] = #10 then begin
            //,  .
            result := true;
          end
          else begin
            //     .
            //  1 .
            TextFil1.FileString := TextFil1.FileString+#13+#10;
            TextFil1.Cursor := TextFil1.Cursor+2;
            result := true;
          end;
        end
        else begin
          //     .
          TextFil1.Cursor := TempCur;  //   ,     
          buf := TextFil1.CurReadStr;  //    +  .

          //  -    .
          // ,  ,     1  .
          //  -     .
          If Length(buf) > 0 then begin
            // ""  1    .
            TextFil1.InsertString(TextFil1.Cursor,#13+#10);
            TextFil1.Cursor := TextFil1.Cursor+2;  // .
            result := true;
          end
          else begin
            // ,     .      :)
            result := true;   //    ...
          end;

        end;

      end;
    end;

  end
  else begin
    //,  - .
    //     ,     .
  end;  
end;

Function TAnIniFile.GoToElement(const SectName, ElemName : string) : boolean;
var
  Finished : boolean;
  TempCur, TempPos : int64;
  Buf : string;
begin
  //    .

  // ...
  Finished := false;
  Result := false;
  TempCur := TextFil1.Cursor;  //    , 
  //   .

  //    
  If GoToSection(SectName,2) = true then begin
    // ,    .
    repeat
      If TextFil1.Eof = false then begin
        TempPos := TextFil1.Cursor;  //,    .
        buf := TextFil1.CurReadLn;
        if length(buf) > 0 then begin
          //    .
          If buf[1] <> '[' then begin
            //    .   :)
            If GetElemName(Buf) = ElemName then begin
              //  ,   :)
              TextFil1.Cursor := TempPos;  //     
              Result := true;
              Finished := true;  //   .
            end;
          end
          else begin
            // ,    .  .
            Finished := true;
          end;
        end;
      end
      else begin
        //  ,   .
        Finished := true;
      end;
    until Finished = true;
  end;

  If Result = false then begin
    //     .
    TextFil1.Cursor := TempCur;
  end;
  
end;

Function TAnIniFile.NotComment(const Str : string) : boolean;

Function EstRavno(const Str : string) : boolean;
var
  I, StrLength : integer;
begin
  //,      ,
  //  ,    .

  Result := false;

  //   (  2- ).
  StrLength := Length(Str);
  If StrLength >= 2 then begin
    // .
    I := 1;  //   2- .
    repeat
      I := I+1;
      If I <= StrLength then begin
        If Str[I] = '=' then begin
          //    .  .
          //      .
          result := true;
        end;
      end;
    until (I > StrLength) or (Str[I] = '=');
  end;
end;

begin
  // ,   .   
  //..    "//",   
  //      ,
  // -    -  :).    
  // .

  Result := false;  //   -  
  //      .
  If Length(Str) > 0 then begin

    //   
    If Str[1] <> '/' then begin
      Result := EstRavno(Str);  //     .
    end
    else begin

      //    2- ,    .
      If Length(Str) > 1 then begin
        If Str[2] <> '/' then begin
          Result := EstRavno(Str);  // .
        end;
      end;

    end;

  end;
  
end;

Function TAnIniFile.GetElemName(const Str : string) : string;
var
  TempStr : string;
begin
  //-   (,    ).
  //   ,   ,  ,
  //..      ,  
  //   ,     .

  Result := '';  //    .
  If NotComment(Str) = true then begin
    //    .
    //   -    :)

    TempStr := Str;
    Result := Parse(TempStr,'=');
  end;
end;

Function TAnIniFile.GetElemStr(const Str : string) : string;
var
  TempStr : string;
begin
  //-   (  ).
  //       ,
  //      (   ).

  TempStr := Str;
  Parse(TempStr,'=');
  Result := TempStr;
end;

Function TAnIniFile.CheckStrInt(const St : string) : boolean;
var
  ch : char;
  I : integer;
begin
  If (st <> '') and (st<>'-') then begin
    result := true;
    I := 0;
    If St[I+1] = '-' then I := I+1;
    repeat
      I := I+1;
      Ch := St[I];
      if ch <> #0 then if CheckCharNum(Ch) = false then result := false;
    until Ch = #0;
  end  
  else result := false;
end;

Function TAnIniFile.CheckCharNum(const Ch : char) : boolean;
begin
  result := false;
  if Ch = '0' then result := true;
  if Ch = '1' then result := true;
  if Ch = '2' then result := true;
  if Ch = '3' then result := true;
  if Ch = '4' then result := true;
  if Ch = '5' then result := true;
  if Ch = '6' then result := true;
  if Ch = '7' then result := true;
  if Ch = '8' then result := true;
  if Ch = '9' then result := true;
end;

Function TAnIniFile.CheckStrFloat(const St : string) : boolean;
var
  zapatih, I : integer;
  ch : char;
begin
  If (st <> '') and (st <> 'NAN') and (st<>'-') then begin
    result := true;
    zapatih := 1;
    I := 0;
    If St[I+1] = '-' then I := I+1;
    repeat
      I := I+1;
      Ch := St[I];
      If ch = '.' then zapatih := zapatih-1
      else if ch <> #0 then if CheckCharNum(Ch) = false then result := false;
    until Ch = #0;
    if zapatih < 0 then result := false;
  end
  else begin
    If St = 'NAN' then result := true
    else Result := false;
  end;
end;

Procedure TAnIniFile.MakNewFile;
begin
  //"  " ;)
  //   -  TextFil1

  TextFil1.MakNewFile;
end;

end.
