////////////////////////////////////////////////////////////
//               KOLFileInString        //
//                -       //
//            ,    .       //
//                        v 1.1                           //
//                                                        //
//              Copyright (C) 2007 Sagrer                 //
//             LGPL 2.1         //
//                  .  lgpl.txt                     //
//                                                        //
//                  sagrer@yandex.ru                      //
////////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

unit KOLTextFileInString;

interface
uses KOL, KOLFileInString, Windows;

type
  TTextFileInString = class (TFileInString)
    //       TFileInString
    Cursor : int64;
    Procedure AddStr(const Str : string);
    Function ReadStrLn(const FirstByte : int64) : string;
    Function CurReadStr : string;
    Function CurReadLn : string;
    Function Eof : boolean;
    Function CurRead(const BytesRead : int64) : string;
    Procedure InsertString(const Pos : int64; Str : string);
    Procedure ReplaceString(const Pos : int64; Str : string);
  end;

implementation

Procedure TTextFileInString.AddStr(const Str : string);
begin
  FileString := FileString+Str+#13#10;
end;

Function TTextFileInString.ReadStrLn(const FirstByte : int64) : string;
var
  I, TempInt : int64;
begin
  //     .
  //  .

  //  -   .
  I := FirstByte;
  TempInt := Length(FileString);
  repeat
    If (FileString[I] <> #13) and (I <> TempInt+1) then begin
      I := I+1;
    end;
  until (FileString[I] = #13) or (I = TempInt+1);

  //    (   ).
  result := ReadStr(FirstByte,I-FirstByte);
end;

Function TTextFileInString.CurReadStr : string;
begin
  //     ,      .
  Result := ReadStrLn(Cursor);
  Cursor := Cursor+Length(Result);
end;

Function TTextFileInString.CurReadLn : string;
begin
  //     ,      . ,
  //   .
  Result := CurReadStr;
  If (Cursor <> Length(FileString)+1) and ((Length(FileString)+1)  - (Cursor) >=2)   then begin
    //     ,    2 , ...
    Cursor := Cursor+2;
  end;
end;

Function TTextFileInString.Eof : boolean;
begin
  //     ,   ...

  result := false;  // ...

  If Cursor = Length(FileString)+1 then begin
    result := true;
  end
  else begin
    If Cursor > Length(FileString)+1 then begin
      //      ,
      // ,   ...
      Cursor := Length(FileString)+1;
      result := true;
    end;
  end;
end;

Function TTextFileInString.CurRead(const BytesRead : int64) : string;
begin
  //,        ,
  //   .

  result := ReadStr(Cursor,BytesRead);
  Cursor := Cursor+BytesRead;
end;

Procedure TTextFileInString.InsertString(const Pos : int64; Str : string);
var
  TempStr : string;
begin
  //,   (    -  
  // -   ) .

  TempStr := ReadStr(1,Pos-1); //     .
  TempStr := TempStr+Str;  // ...
  If Pos-1 <> Length(FileString) then begin
    //    ...
    TempStr := TempStr+ReadStr(Pos,Length(FileString)-Pos+1);  //  .
  end;  
  FileString := TempStr;
end;

Procedure TTextFileInString.ReplaceString(const Pos : int64; Str : string);
var
  TempStr, buf : string;
  TempPos : int64;
begin
  //   (    )
  //   (  #13#10     ...)
  TempStr := ReadStr(1,Pos-1); //     .
  TempStr := TempStr+Str;  // ...
  If Pos-1 <> Length(FileString) then begin
    //    ...
    buf := ReadStrLn(Pos);    //  ,    
    //  ,    .
    TempPos := Pos+Length(buf);

    TempStr := TempStr+ReadStr(TempPos,Length(FileString)-TempPos+1);  //  .
  end;
  FileString := TempStr;
end;

end.
